// Arduija 2.0 
// by: Joel Bartlett
// SparkFun Electronics
// date: October 8, 2012
// license: BeerWare
//          Please use, reuse, and modify this code as you need.
//          We hope it saves you some time, or helps you learn something!
//          If you find it handy, and we meet some day, you can buy me a beer or iced tea in return.

/* This code takes input from a Wii nunchuck over I2C. 
It then uses this data to controller to stepper motors 
hooked up to Easy Driver motor driver boards. This code 
heavily borrows from the code available on the Wii chuck 
adapter homepage 
http://todbot.com/blog/2008/02/18/wiichuck-wii-nunchuck-adapter-available/
It also borrows from the stepper motor example code found on the bilder website
http://bildr.org/?s=stepper
*/

#include <Wire.h>  //include the Wire Library for communicating to the WiiChuck
#include <string.h>  //include the string library to handle the data from the WiiChuck

uint8_t outbuf[6];// array to store arduino output
int cnt = 0;//var to count the number of bytes

int stepperPin = 2;  //motor 1 declarations
int dirPin = 3;
int enable = 4;

int stepperPin2 = 8; //motor 2 
int dirPin2 = 9;
int enable2 = 10;

int joy_x_axis = 125;//variable for the x axis on the joystick. 125 is the center value. 
//It is set to this initially to avoid triggering the motors before the first reading from the WiiChuck can be transmitted
int joy_y_axis = 125;//variable for the y axis on the joystick.

int z_button; //these two button aren't used here, but you could add functionality with them later. 
int c_button;
//-------------------------------------------------------------------
void setup ()
{
  //Serial.begin(9600);
  Wire.begin ();// join i2c bus with address 0x52
  nunchuck_init (); // send the initilization handshake
  
  pinMode(dirPin, OUTPUT); //declare all of our outputs 
  pinMode(stepperPin, OUTPUT);
  pinMode(enable, OUTPUT);
  
  pinMode(dirPin2, OUTPUT);
  pinMode(stepperPin2, OUTPUT);
  pinMode(enable2, OUTPUT);
  
  digitalWrite(enable, HIGH); ///NOT enabled. The motor is of and spins freely
  digitalWrite(enable2, HIGH); ///NOT enabled
  WiiRequest(); //added this because the motor was spinning one revolution upon startup
                //haveing this makes it so the motor dosn't spin before it can get the first request from the chuck. 
}
//-----------------------------------------------------------------
void nunchuck_init ()// initializes the WiiChuck
{
  Wire.beginTransmission (0x52);	// transmit to device 0x52
  Wire.send (0x40);		// sends memory address
  Wire.send (0x00);		// sends sent a zero.  
  Wire.endTransmission ();	// stop transmitting
}
//-------------------------------------------------------------------
void send_zero ()// sends the byte 0x00
{
  Wire.beginTransmission (0x52);	// transmit to device 0x52
  Wire.send (0x00);		// sends one byte
  Wire.endTransmission ();	// stop transmitting
}
//-------------------------------------------------------------------
void loop ()
{
 
  WiiRequest(); // get data from the WiiChuck
  int joy_x_axis = outbuf[0]; //the first byte is the position of the x-axis
  int joy_y_axis = outbuf[1]; // the next byte is the y axis
  //Serial.println(joy_x_axis);// print the values to the serial terminal
  //Serial.println(joy_y_axis);
  
  //Now we check to the position of both the x and y axis 
  //and tell the motors to spin in the corresponding directions
  if(joy_x_axis < 75 && joy_y_axis < 175 && joy_y_axis > 75)  //move left X
  {
    digitalWrite(enable, LOW);
    //rotate a specific number of degrees 
    rotateDeg(360, .65);
    //delay(1000);
  }
  else if(joy_x_axis > 175 && joy_y_axis < 175 && joy_y_axis > 75) //move right X
    {
    digitalWrite(enable, LOW);
   //rotate a specific number of degrees 
    rotateDeg(-360, .65);
    //delay(1000);
  }
   else if(joy_y_axis < 75 && joy_x_axis < 175 && joy_x_axis > 75)  //move down Y
  {
    digitalWrite(enable2, LOW);
    //rotate a specific number of degrees 
    rotateDeg2(360, .65);
    //delay(1000);
  }
  else if(joy_y_axis > 175  && joy_x_axis < 175 && joy_x_axis > 75) //move up Y
    {
    digitalWrite(enable2, LOW);
   //rotate a specific number of degrees 
    rotateDeg2(-360, .65);
    //delay(1000);
  }
   else if(joy_x_axis < 75 && joy_y_axis > 175)  //move left and up
  {
    digitalWrite(enable, LOW);
    digitalWrite(enable2, LOW);
    rotateDegAll(360, .65, -360);
    //delay(1000);
  }
     else if(joy_x_axis < 75 && joy_y_axis < 75)  //move left and down 
  {
    digitalWrite(enable, LOW);
    digitalWrite(enable2, LOW);
    rotateDegAll(360, .65, 360);
    //delay(1000);
  }
     else if(joy_x_axis >175 && joy_y_axis > 175)  //move right and up
  {
    digitalWrite(enable, LOW);
    digitalWrite(enable2, LOW);
    rotateDegAll(-360, .65, -360);
    //delay(1000);
  }
     else if(joy_x_axis > 175 && joy_y_axis < 75)  //move right and down 
  {
    digitalWrite(enable, LOW);
    digitalWrite(enable2, LOW);
    rotateDegAll(-360, .65, 360);
    //delay(1000);
  }
   else
  {
    digitalWrite(enable, HIGH);
    digitalWrite(enable2, HIGH); ///NOT enabled
  }
  
  


  
}
//--------------------------------------------------------------------
void WiiRequest()
{
   Wire.requestFrom (0x52, 6);	// request data from nunchuck
  while (Wire.available ())
    {
      outbuf[cnt] = nunchuk_decode_byte (Wire.receive ());	// receive byte as an integer
      //digitalWrite (ledPin, HIGH);	// sets the LED on
      cnt++;
    }
  // If we recieved the 6 bytes, then go print them

  //if (cnt >= 5)
    //{
      //print (); // took this function out, was used ot print all the values at once
    //}

  cnt = 0;
  send_zero (); // send the request for next bytes
  delay (10);
}
//------------------------------------------------------------------------
// Encode data to format that most wiimote drivers except
// only needed if you use one of the regular wiimote drivers
char nunchuk_decode_byte (char x)
{
  x = (x ^ 0x17) + 0x17;
  return x;
}
//---------------------------------------------------------------
void rotateDeg(float deg, float speed)// this function takes a variable for how many degrees to move and a var for speed
//use this function for the x-axis motor
{
  //rotate a specific number of degrees (negitive for reverse movement)
  //speed is any number from .01 -> 1 with 1 being fastest - Slower is stronger
  int dir = (deg > 0)? HIGH:LOW;
  digitalWrite(dirPin,dir); 

  int steps = abs(deg)*(1/0.225);
  float usDelay = (1/speed) * 70;

  //do
    for(int i=0; i < steps; i++)
  {
    digitalWrite(stepperPin, HIGH);
    delayMicroseconds(usDelay); 

    digitalWrite(stepperPin, LOW);
    delayMicroseconds(usDelay);
  }//while(joy_x_axis <= 75 || joy_x_axis >= 175);
}
//---------------------------------------------------------------
void rotateDeg2(float deg, float speed)//use this function for the y-axis motor
{
  //rotate a specific number of degrees (negitive for reverse movement)
  //speed is any number from .01 -> 1 with 1 being fastest - Slower is stronger
  int dir = (deg > 0)? HIGH:LOW;
  digitalWrite(dirPin2,dir); 

  int steps = abs(deg)*(1/0.225);
  float usDelay = (1/speed) * 70;

  //do
    for(int i=0; i < steps; i++)
  {
    digitalWrite(stepperPin2, HIGH);
    delayMicroseconds(usDelay); 

    digitalWrite(stepperPin2, LOW);
    delayMicroseconds(usDelay);
  }//while(joy_x_axis <= 75 || joy_x_axis >= 175);
}
//---------------------------------------------------------------
void rotateDegAll(float deg, float speed, float deg2)// use this function to rotate both motors at the same time
//really it moves one after the other. Still need to figure out how to turn both on at the same time. 
{
   //rotate a specific number of degrees (negitive for reverse movement)
  //speed is any number from .01 -> 1 with 1 being fastest - Slower is stronger
  int dir = (deg > 0)? HIGH:LOW;
  digitalWrite(dirPin,dir); 
  
  int dir2 = (deg2 > 0)? HIGH:LOW;
  digitalWrite(dirPin2,dir2); 

  int steps = abs(deg)*(1/0.225);
  float usDelay = (1/speed) * 70;
  
  int steps2 = abs(deg2)*(1/0.225);
  float usDelay2 = (1/speed) * 70;

    for(int i=0; i < steps; i++)
  {
    digitalWrite(stepperPin, HIGH);
    delayMicroseconds(usDelay); 

    digitalWrite(stepperPin, LOW);
    delayMicroseconds(usDelay);
  }
  for(int i=0; i < steps2; i++)
  {
    digitalWrite(stepperPin2, HIGH);
    delayMicroseconds(usDelay2); 

    digitalWrite(stepperPin2, LOW);
    delayMicroseconds(usDelay2);
  }
}
